@@ -2,9 +2,11 @@
 #'
 #' Read climate (meta) data that was downloaded with \code{\link{dataDWD}}.
 #' The file is read, processed and returned as a data.frame.\cr
-#' If \code{meta=TRUE}: column widths for \code{\link{read.fwf}} are computed internally.\cr
 #' \code{file} can be a vector with several filenames. The arguments \code{meta}
-#' and \code{format} can also be a vecor and will be recycled to the length of \code{file}.
+#' and \code{format} can also be a vecor and will be recycled to the length of \code{file}.\cr
+#' If \code{meta=TRUE}, column widths for \code{\link{read.fwf}} are computed internally.
+#' If needed, readDWD tries to set the locale to German (to handle Umlaute correctly).
+#' They can then be processed with \code{dd$Stations_id <- berryFunctions::convertUmlaut(dd$Stations_id)}.
 #'
 #' @return Invisible data.frame of the desired dataset, or a list of data.frames
 #'         if length(file) > 1.
@@ -12,19 +14,22 @@
 #' @seealso \code{\link{dataDWD}}, \code{\link{selectDWD}}
 #' @keywords file chron
 #' @importFrom utils read.table unzip read.fwf
-#' @importFrom berryFunctions checkFile na9
+#' @importFrom berryFunctions checkFile na9 traceCall
 #' @export
 #' @examples
 #' # see dataDWD
 #'
 #' @param file   Char (vector): name(s) of the file(s) downloaded with \code{\link{dataDWD}},
-#'               e.g. "tageswerte_KL_02575_akt.zip" or
-#'               "RR_Stundenwerte_Beschreibung_Stationen.txt"
-#' @param dir    Char: directory name where to read the file. Use \code{"."} or \code{""}
-#'               if \code{file} already includes a (relative or absolute) path.
-#'               DEFAULT: "DWDdata" at current \code{\link{getwd}()}
+#'               e.g. "DWDdata/tageswerte_KL_02575_akt.zip" or
+#'               "DWDdata/RR_Stundenwerte_Beschreibung_Stationen.txt"
 #' @param meta   Logical (vector): is the \code{file} a meta file?
 #'               DEFAULT: TRUE for each file ending in ".txt"
+#' @param fread  Logical: read faster with \code{data.table::\link[data.table]{fread}}?
+#'               For 30 daily/kl/hist files, 7 instead of 10 seconds.
+#'               DEFAULT: TRUE if data.table is available.
+#' @param html   Logical: read the html file instead of actual data?
+#'               Requires XML to be installed. Experimental: use at own risk!
+#'               DEFAULT: FALSE
 #' @param format Char (vector): if \code{meta=FALSE}: Format passed to
 #'               \code{\link{as.POSIXct}} (see \code{\link{strptime}})
 #'               to convert the date/time column to POSIX time format.\cr
@@ -36,12 +41,13 @@
 #'               Coordinated). DEFAULT: "GMT"
 #' @param progbar Logical: present a progress bar with estimated remaining time?
 #'               If missing and length(file)==1, progbar is internally set to FALSE.
-#'               Only works if the R package \code{pbapply} is available. DEFAULT: TRUE 
+#'               Only works if the R package \code{pbapply} is available. DEFAULT: TRUE
 #'
 readDWD <- function(
 file,
-dir="DWDdata",
 meta=substr(file, nchar(file)-3, 1e4)==".txt",
+fread=requireNamespace("data.table",quietly=TRUE),
+html=FALSE,
 format=NA,
 tz="GMT",
 progbar=TRUE
@@ -53,15 +59,16 @@ if(missing(progbar) & len==1) progbar <- FALSE
 if(len>1)
   {
   meta   <- rep(meta,   length.out=len)
+  fread  <- rep(fread, length.out=len)
+  html   <- rep(html,   length.out=len)
   format <- rep(format, length.out=len)
   tz     <- rep(tz,     length.out=len)
   }
-# set directory from which to read downloaded data
-owd <- dirDWD(dir, quiet=TRUE)
-on.exit(setwd(owd))
 # Optional progress bar:
-progbar <- progbar & requireNamespace("pbapply", quietly=TRUE)
 if(progbar) lapply <- pbapply::pblapply
+if(any(fread))   if(!requireNamespace("data.table", quietly=TRUE))
+     stop("in readDWD: Please first install data.table:",
+          "   install.packages('data.table')", call.=FALSE)
 #
 checkFile(file)
 # loop over each filename
@@ -70,22 +77,43 @@ output <- lapply(seq_along(file), function(i)
 f <- file[i]
 if(!meta[i]) # if data ---------------------------------------------------------
 {
+if(html[i]) fread[i] <- FALSE
+if(fread[i]) # http://dsnotes.com/post/2017-01-27-lessons-learned-from-outbrain-click-prediction-kaggle-competition/
+{
+  fp <- unzip(f, list=T)
+  fp <- fp$Name[grepl("produkt",fp$Name)]
+  dat <- data.table::fread(paste("unzip -p", f, fp), na.strings=na9(),
+                           header=TRUE, sep=";", stringsAsFactors=TRUE)
+  dat <- as.data.frame(dat) # remove data.table class
+} else
+{
 # temporary unzipping directory
-exdir <- paste0(tempdir(),"/", substr(f, 1, nchar(f)-4))
+fn <- tools::file_path_sans_ext(basename(f))
+exdir <- paste0(tempdir(),"/", fn)
 unzip(f, exdir=exdir)
-on.exit(unlink(exdir, recursive=TRUE))
+on.exit(unlink(exdir, recursive=TRUE), add=TRUE)
+# experimental: html meta info:
+if(html[i])
+{
+  if(!requireNamespace("XML", quietly=TRUE))
+    stop("in readDWD: Please first install XML:   install.packages('XML')", call.=FALSE)
+  f <- dir(exdir, pattern="*html", full.names=TRUE)
+  tabs <- lapply(f, function(fi)
+    {
+    tab <- XML::readHTMLTable(fi, stringsAsFactors=FALSE)[[1]]
+    tab <- paste(apply(tab,1,paste,collapse=";"), collapse="\n")
+    tab <- read.table(header=TRUE, sep=";",stringsAsFactors=FALSE, text=tab)
+    tab
+    })
+  names(tabs) <- basename(f)
+  return(tabs)
+}
 # new filename - the actual data file:
 f <- dir(exdir, pattern="produkt*", full.names=TRUE)
 # Actually read data
 dat <- read.table(f, na.strings=na9(), header=TRUE, sep=";", as.is=FALSE)
-##
-## # The alternative would be to use
-## fn <- unzip(f, list=TRUE)
-## fn <- fn$Name[grep(pattern="produkt", x=fn$Name)]
-## dat <- read.table(unz(f,fn), na.strings=na9(), header=TRUE, sep=";", as.is=FALSE)
-## # but it is not faster and introduces incomplete NA rows at the data.frame end
-## # and also removes the opportunity to potentially add an argument cleanup=FALSE
-##
+} # end if not fread
+#
 # process time-stamp: http://stackoverflow.com/a/13022441
 if(!is.null(format[i]) & "MESS_DATUM" %in% colnames(dat))
   {
@@ -96,9 +124,18 @@ if(!is.null(format[i]) & "MESS_DATUM" %in% colnames(dat))
 return(dat)
 } else # if meta ---------------------------------------------------------------
 {
+# Handle German Umlaute:
+lct <- Sys.getlocale("LC_CTYPE")
+on.exit(Sys.setlocale(category="LC_CTYPE", locale=lct), add=TRUE)
+if(!grepl(pattern="german", lct, ignore.case=TRUE))
+  {
+  lctry <- c("German","de_DE","de_DE.UTF-8","de_DE.utf8","de")
+  for(lc in lctry) if(suppressWarnings(Sys.setlocale("LC_CTYPE", lc))!="") break
+  }
+#
 # read one line to get column widths and names
 oneline <- readLines(f, n=3)
-# column widhts (automatic detection across different styles used by the DWD)
+# column widths (automatic detection across different styles used by the DWD)
 spaces <- unlist(gregexpr(" ", oneline[3]))
 breaks <- spaces[which(diff(spaces)!=1)]
 if(substr(oneline[3],1,1)==" ") breaks <- breaks[-1]
@@ -118,7 +155,7 @@ if(!all(actual == classes))
   msg <- paste0(names(actual)[actual!=classes], ": ", actual[actual!=classes],
                 " instead of ", classes[actual!=classes], ".")
   msg <- paste(msg, collapse=" ")
-  warning("in rdwd::readDWD: reading file '", f,
+  warning(traceCall(3, "", ": "), "reading file '", f,
           "' did not give the correct column classes. ", msg, call.=FALSE)
   }
 # return meta data.frame:
